/*------------------------------------------------------------------------------

 Author:  Michael Barfs, IAP GmbH, mb@iap.de
 Created: March 2018
 Puprose: Demo code for OOABL pattern presentation
          This object demostrates the Multiton Pattern

 Notes:
  - This class uses itself, therefore the class itself must be referenced by using
  - getInstance and Temp-Table are _STATIC_
  - The constructor is private, so the getInstance must be used

------------------------------------------------------------------------------*/

USING 03_multiton.multiCust.

CLASS 03_multiton.multiCust:


  // Data properties of class
  DEFINE PUBLIC PROPERTY iCustNum AS INTEGER NO-UNDO
    PUBLIC GET.
    PRIVATE SET.
  DEFINE PUBLIC PROPERTY cCustName AS CHARACTER NO-UNDO
    PUBLIC GET.
    PRIVATE SET.

  /* This temp-table holds handles to record buffers, not to data. */
  DEFINE PRIVATE STATIC TEMP-TABLE ttCustomer NO-UNDO
    FIELD iNumber  AS INTEGER
    FIELD oObject  AS Progress.Lang.Object
    INDEX ID iNumber.


  /* The constructor will be fired for every customer which is requested. 
     This will work whether there is a cust record or not. */
  CONSTRUCTOR PRIVATE multiCust(iCustNum AS INTEGER):

    DEFINE BUFFER bCustomer FOR Customer.
    FIND FIRST bCustomer WHERE bCustomer.CustNum = iCustNum NO-LOCK NO-ERROR.
    IF AVAILABLE bCustomer THEN DO:
      THIS-OBJECT:cCustName  = bCustomer.Name.
      THIS-OBJECT:iCustNum   = bCustomer.custNum.
    END.

  END CONSTRUCTOR.

  /* This STATOC method looks for temp-table entry. If not avail it will create a
     dynamic object and store the reference in the temp-table. */
  METHOD PUBLIC STATIC multiCust getInstance(iCustNum AS INTEGER):

    FIND FIRST ttCustomer WHERE ttCustomer.iNumber = iCustNum NO-LOCK NO-ERROR.
    IF NOT AVAILABLE ttCustomer THEN DO:
      CREATE ttCustomer.
      ASSIGN
        ttCustomer.iNumber = iCustNum
        ttCustomer.oObject = NEW multiCust(iCustNum).
    END.
    RETURN CAST(ttCustomer.oObject, multiCust).

  END METHOD.

END CLASS.
