 /*------------------------------------------------------------------------
    File        : ghostCustomer
    Author(s)   : michael.barfs
    Created     : Mon Feb 26 14:50:42 CET 2018

  The ghost pattern uses a holder (iID) and when data are accessed, 
  the real data are fetched from the DB - see InitializeIfNecessary.  
  ----------------------------------------------------------------------*/

USING Progress.Lang.*.

CLASS 04_lazy_loading.ghost.ghostCustomer:

/*---------------------------------Definitions----------------------------------*/

  DEFINE BUFFER bCustomer FOR customer.

  DEFINE PRIVATE PROPERTY iID AS INTEGER NO-UNDO
    PRIVATE GET.
    PRIVATE SET.

  DEFINE PRIVATE PROPERTY lLoaded AS LOGICAL NO-UNDO
    PRIVATE GET.
    PRIVATE SET.

  DEFINE PUBLIC PROPERTY cName AS CHARACTER NO-UNDO
    PUBLIC GET():
      InitializeIfNecessary().
      RETURN cName.
    END GET.
    PRIVATE SET.

  DEFINE PUBLIC PROPERTY cAddress AS CHARACTER NO-UNDO
    PUBLIC GET():
      InitializeIfNecessary().
      RETURN cAddress.
    END GET.
    PRIVATE SET.

/*-------------------------------- Constructor----------------------------------*/

  CONSTRUCTOR PUBLIC ghostCustomer(iID AS INTEGER):
    THIS-OBJECT:iID = iID.
  END CONSTRUCTOR.

/*-----------------------------------Method-------------------------------------*/

  METHOD PRIVATE VOID InitializeIfNecessary():
    IF NOT lLoaded THEN DO:
      FIND FIRST bCustomer WHERE bCustomer.CustNum = iID NO-LOCK NO-ERROR.
      IF AVAILABLE bCustomer THEN DO:
        cName    = bCustomer.Name.
        cAddress = bCustomer.Address.
      END.
    END.
  END METHOD.

/*----------------------------------End Class-----------------------------------*/

END CLASS.